/*
 * device driver for philips saa7134 based TV cards
 * gpio interface support
 *
 * based on bttv-if.c (part of bttv - bt848 frame grabber driver)
 *
 * Copyright (C) 1996,97,98 Ralph  Metzler (rjkm@thp.uni-koeln.de)
 *                        & Marcus Metzler (mocm@thp.uni-koeln.de)
 * (c) 1999-2003 Gerd Knorr <kraxel@bytesex.org>
 * (c) 2003 David Atkinson <px-gpio@psykax.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/init.h>
#include <linux/list.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/delay.h>

#include "saa7134-reg.h"
#include "saa7134.h"
#include "saa7134-gpio.h"

EXPORT_SYMBOL(saa7134_get_cardinfo);
EXPORT_SYMBOL(saa7134_get_pcidev);
EXPORT_SYMBOL(saa7134_gpio_enable);
EXPORT_SYMBOL(saa7134_read_gpio);
EXPORT_SYMBOL(saa7134_write_gpio);
EXPORT_SYMBOL(saa7134_get_gpio_queue);

extern unsigned int gpio_tracking;

struct saa7134_dev* get_device(unsigned int card)
{
	/* Gets a pointer to a saa7134_dev struct from
	   a card index */

	unsigned int list_count = 0;
	struct list_head* list_ptr;

	list_for_each(list_ptr, &saa7134_devlist) {
		if(list_count == card) break;
		list_count ++;
	}

	return (struct saa7134_dev*)list_ptr;
}

/* ----------------------------------------------------------------------- */
/* Exported functions - for other modules which want to access the         */
/*                      gpio ports (IR for example)                        */

int saa7134_get_cardinfo(unsigned int card, unsigned int *board, unsigned long *pci_subsystem)
{
	struct saa7134_dev* dev;
	dev = get_device(card);

	*board  = dev->board;
	*pci_subsystem = (dev->pci->subsystem_vendor << 16) & (dev->pci->subsystem_device);

	return 0;
}

struct pci_dev* saa7134_get_pcidev(unsigned int card)
{
	return get_device(card)->pci;
}

int saa7134_gpio_enable(unsigned int card, unsigned long mask, unsigned long data)
{
	struct saa7134_dev* dev;
	dev = get_device(card);

	/* TODO: Is this needed? */

	/* if(gpio_tracking) saa7134_track_gpio(dev,"extern enable"); */
	return 0;
}

int saa7134_read_gpio(unsigned int card, unsigned long *data)
{
	struct saa7134_dev* dev;
	dev = get_device(card);

	/* rising SAA7134_GPIO_GPRESCAN reads the status */
	saa_andorb(SAA7134_GPIO_GPMODE3,SAA7134_GPIO_GPRESCAN,0);
	saa_andorb(SAA7134_GPIO_GPMODE3,SAA7134_GPIO_GPRESCAN,SAA7134_GPIO_GPRESCAN);
	/* mode   = saa_readl(SAA7134_GPIO_GPMODE0   >> 2) & 0xfffffff; */
	*data = saa_readl(SAA7134_GPIO_GPSTATUS0 >> 2) & 0xfffffff;
	return 0;
}

int saa7134_write_gpio(unsigned int card, unsigned long mask, unsigned long data)
{
	struct saa7134_dev* dev;
	dev = get_device(card);

	/* TODO: Implement this */

	if (gpio_tracking) saa7134_track_gpio(dev,"extern write");
	return 0;
}

wait_queue_head_t* saa7134_get_gpio_queue(unsigned int card)
{
	struct saa7134_dev* dev;
	dev = get_device(card);

	/* TODO: Is this needed? */

	/* return &dev->gpioq; */
	return 0;
}
